import { utils, mapViewUtils } from "@/common/_utils/gis-utils";
import { uploadSHP, readTxT, readJSON, featureCollection2GeometryCollection } from "@/common/_utils/util";
import WebMapService from "@/common/_utils/webmapservice";
import { formatKQMapInfo } from "@/common/_utils/util";
import XEUtils from "xe-utils";
import { KqMessage } from "@/common";

export default class AddDataViewModel extends L.Evented {
  _props = null;
  webMap = null;
  constructor(mapTarget, geometryServerUrl, geometryServerToken) {
    super();
    this._props = { mapTarget, geometryServerUrl, geometryServerToken };

    this._loadedAsync = this._loadedAsync.bind(this);
    this._loadedSync = this._loadedSync.bind(this);
    this._registerEvents();
  }

  _registerEvents() {
    // 监听地图初始化完成事件
    // 异步
    mapViewUtils.bindViewEvents("mounted", this._loadedAsync, this._props.mapTarget);
    // 同步
    utils.getWebMap(this._props.mapTarget, this._loadedSync);
  }

  _loadedAsync(e) {
    // 如果图层变化事件是自己触发的 则不继续执行 避免死循环
    if (e.eventTarget && e.eventTarget === this) {
      return;
    }
    this.webMap = e.target;
  }

  _loadedSync(e) {
    this.webMap = e;
  }

  // 单个导入文件
  uploadFile(file) {
    return new Promise((resolve, reject) => {
      if (file.status === "ready") {
        let fileType = file.name.substring(file.name.lastIndexOf("."));
        if (fileType.toUpperCase() === ".ZIP" || fileType.toUpperCase() === ".RAR") {
          uploadSHP(this._props.geometryServerUrl, file.raw, this._props.geometryServerToken)
            .then(res => {
              this._addMapLayer(res, file.name);
              resolve("success");
            })
            .catch(err => {
              reject(file.uid);
            });
        } else if (fileType.toUpperCase() === ".TXT") {
          readTxT(file.raw, res => {
            if (res) {
              this._addMapLayer(res, file.name);
              resolve("success");
            } else {
              reject(file.uid);
            }
          });
        } else if (fileType.toUpperCase() === ".JSON" || fileType.toUpperCase() === ".GEOJSON") {
          readJSON(file.raw, res => {
            if (res) {
              this._addMapLayer(res, file.name);
              resolve("success");
            } else {
              reject(file.uid);
            }
          });
        }
      }
    });
  }
  // 批量导入文件
  uploadFiles(files) {
    var arr = [];
    files.forEach(file => {
      arr.push(this.uploadFile(file));
    });
    return Promise.all(arr);
  }

  // 添加图层到地图
  _addMapLayer(geojson, filename) {
    let gc = featureCollection2GeometryCollection(geojson);
    var layers = [
      {
        name: filename.split(".")[0],
        guid: XEUtils.uniqueId("addData_"),
        data: gc
      }
    ];
    this.webMap.addTempData(layers, true);
  }

  /**
   * URL添加数据
   * @param {string} type
   * @param {string} url - 服务地址，例：http://support.kqgeo.com:29800/china3857-proxy/kqgis/rest/services/china3857/map
   * @param {string} [name] - 服务图层名称
   */
  loadURL(params) {    
    return new Promise((resolve, reject) => {
      var serverName = params.name ? params.name : params.url.split("services/")[1].split("/")[0];
      var serverUrl = params.url.split("services/")[0] + "services/";    
      let webMapService = new WebMapService(serverName, {
        token: this._props.geometryServerToken,
        serverUrl: serverUrl
      });    
      webMapService.getMapInfo().then(
        mapInfo => {
          if (mapInfo.resultcode === "error") {
            reject(error);
            KqMessage({ message: mapInfo.message, grouping: true, type: "error", duration: 6000 });
            return;
          }
          // 存储地图的名称以及描述等信息，返回给用户
          const webMapInfo = formatKQMapInfo(
            mapInfo,
            serverUrl,
            serverName,
            this._props.geometryServerToken,
            null,
            true
          );
          this.webMap.addTempData(webMapInfo.layerTreeData, true, webMapInfo.layers_show);
          console.log(this.webMap);
          resolve(webMapInfo);
        },
        error => {
          throw error;
        }
      );
    });
  }

  destroy() {
    this.off();
  }
}
