import swipe from "./libs/swipe";

export default class MapControl {
  map = null;
  miniMap = null;
  // 滤镜配置
  filterConfig = {
    radius: 100,
    zoomOffset: 3,
    fixedZoom: -1,
    fixedPosition: false
  };
  // 比例尺配置
  scaleConfig = {
    fill: false,
    doubleLine: false,
    showSubunits: false,
    minUnitWidth: 30,
    maxUnitsWidth: 240,
    labelPlacement: "auto",
    position: "bottomleft",
    updateWhenIdle: false
  };
  constructor(mapView) {
    // 地图控件
    this.map = mapView.map;
    // 绑定的地图id
    this.mapTarget = mapView.target;
    this._mapView = mapView;
    // 卷帘选择图层dom节点
    this.rollerLayerDiv = null;
  }

  // 销毁缓存
  destroy() {
    this.map = null;
  }

  /**
   * 创建卷帘
   * @param {Object} layer - 选中需要卷帘的图层，不传时会清除卷帘
   */
  initRoller(layer) {
    if (!layer) {
      if (swipe.isSwipe) {
        swipe.stopSwipe();
      }
      return;
    }
    if (swipe.isSwipe) {
      swipe.stopSwipe();
    }
    if (layer.id === 200) return;
    this.map.trackResize = false;
    let selectedOverLayer = this._mapView._getLayer(layer);
    if (selectedOverLayer._currentOverlay !== undefined) {
      let paneName = selectedOverLayer._currentOverlay.options.pane;
      this.rollerLayerDiv = selectedOverLayer.getPane(paneName);
    } else {
      this.rollerLayerDiv = selectedOverLayer.getContainer();
    }
    swipe.startSwipe(this._mapView.target, this._mapView.map, this.rollerLayerDiv);
  }

  /**
   * 生成滤镜
   * @param {Object} config
   * @param {number} [config.radius=100] - 放大镜的半径，以像素为单位
   * @param {number} [config.zoomOffset=3] - 主地图缩放和放大镜之间的缩放级别偏移
   * @param {number} [config.fixedZoom= -1] - 如果不同于-1，则定义始终在放大镜中使用的固定缩放级别，忽略主地图缩放和zoomOffet值。
   * @param {Boolean} [config.fixedPosition= false] - 如果true，放大镜将停留在地图上的同一位置，而不是跟随鼠标光标。
   * @param {array} latLng - 放大镜的初始位置，无论是在主地图上还是作为放大视图的中心。如果fixedPosition是true，它会一直保持这个位置。
   * @param {array} layers - 要在放大视图中显示的图层集
   */
  createFilter(config) {
    config = Object.assign({}, this.filterConfig, config);
    if (config.layers && config.layers.length) {
      let layers = [];
      for (var i = 0, item; (item = config.layers[i++]); ) {
        const newLayer = this._mapView._createLayer(item);
        layers.push(newLayer);
      }
      config.layers = layers;
      var magnifyingGlassLayer = L.magnifyingGlass(config).addTo(this.map);
      return {
        magnifyingGlassLayer
      };
    }
  }

  /**
   * 清除滤镜
   */
  clearFilter(layer) {
    this._mapView.map.removeLayer(layer);
  }

  /**
   * 生成比例尺并返回
   * @param {Object} config
   * @param {string|boolean} [config.fill=false] - 填充，false/'nofill'|'fill'|'hollow'|'line',默认 false
   * @param {boolean} [config.doubleLine=false] - 双线模式，默认false关闭
   * @param {boolean} [config.showSubunits=false] - 显示子单元,在零的左侧显示较小的分区,默认false关闭
   * @param {number} [config.minUnitWidth=30] - 比例单位的最小宽度
   * @param {number} [config.maxUnitsWidth=240] - 不包含子单位的比例尺的最大宽度
   * @param {string} [config.labelPlacement='auto'] - 'auto'|'top'|'bottom'，在比例尺的顶部/底部显示距离标签。如果设置为自动，当比例控制在地图底部时，标签将被放置在顶部，当比例控制在地图顶部时，标签将被放置在底部（position参数）
   * @param {string} [config.position='bottomleft'] - 控件的位置（地图角之一）。可能的值为'topleft', 'topright','bottomleft'或'bottomright'
   * @param {boolean} [config.updateWhenIdle=false] - 如果true，控件在 地图moveend后更新，否则它在地图move时更新。
   */
  createScale(config) {
    config = Object.assign({}, this.scaleConfig, config);
    let graphicScale = window.L.control.graphicScale(config).addTo(this.map);
    return graphicScale;
  }

  /**
   * 开启鹰眼
   * @param layerDatas {Array<Object|string>} object支持mapview的_createLayer方法的参数，string是传入guid,从mapview的_mapLayerListData中找匹配的
   * @param options
   */
  startMiniMap(layerDatas, options) {
    let layerGroup = new L.LayerGroup();
    layerDatas.forEach(item => {
      let layerConfig = null;
      if (typeof item === "String") {
        layerConfig = getData(item);
      } else {
        layerConfig = item;
      }
      if (layerConfig) {
        let layer = this._mapView._createLayer(layerConfig);
        layerGroup.addLayer(layer);
      }
    });
    if (!this.miniMap) {
      this.miniMap = new window.L.Control.MiniMap(layerGroup, options).addTo(this.map);
    } else {
      //this.miniMap.changeLayer(layer);
    }

    function getData(guid) {
      for (let i = 0; i < this._mapView._mapLayerListData.length; i++) {
        if (this._mapView._mapLayerListData[i].guid === guid) {
          return this._mapView._mapLayerListData[i];
        }
      }
      return null;
    }
  }
}
