/*
     绘图相关的操作类：
     根据坐标绘制图形，
     鼠标操作绘制、编辑图形、管理状态，
     编辑图形样式，
     创建存储图形的分组类，根据分组管理图形：添加、删除、获取
     不同坐标格式转换
     根据条件创建图形对象
*/
import markerUrl from "@/../public/images/point_position.png";
export default class MapDraw {
  map = null;
  layerGroups = {};
  areaMeasure = null;

  // 根据坐标绘制的样式
  drawLocationStyle = {
    color: "rgb(0,255,255)",
    fillColor: "rgba(0,255,255, 0.3)",
    weight: 3,
    fillOpacity: 0.5
  };
  // 绘图定位填充样式
  fillLocationStyle = {
    lineColor: "rgba(255, 0, 0, 1)",
    lineWidth: "1px",
    lineType: "left", // left(左斜线)、right(右斜线)、grid(网格线)
    lineSpacing: 12 // 填充线的间距
  };
  // 手动绘图的绘图样式
  drawStyle = {
    color: "#3388ff",
    fillColor: "#3388ff",
    weight: 2,
    fillOpacity: 0.3,
    dashArray: "0"
  };
  // 测量绘图样式
  measureStyle = {
    color: "#3388ff",
    fillColor: "#3388ff",
    weight: 2,
    fillOpacity: 0.1,
    dashArray: "3,4"
  };

  constructor(mapView) {
    // 地图控件
    this.map = mapView.map;
    // 绑定的地图id
    this.mapTarget = mapView.target;
    this.layerGroups = {};
    // 测量对象
    this.areaMeasure = null;
    this._mapView = mapView;

    this.layerGroups["draw"] = window.L.featureGroup().addTo(this.map);

    let _mapviewProps = mapView._props;
    if (_mapviewProps.mapDrawStyle) {
      this.drawStyle = { ...this.drawStyle, ..._mapviewProps.mapDrawStyle };
    }
    if (_mapviewProps.mapDrawLocationStyle) {
      this.drawLocationStyle = { ...this.drawLocationStyle, ..._mapviewProps.mapDrawLocationStyle };
    }
    if (_mapviewProps.mapMeasureStyle) {
      this.measureStyle = { ...this.measureStyle, ..._mapviewProps.mapMeasureStyle };
    }
  }

  // 销毁缓存
  destroy() {
    this.map = null;
  }

  /**
   * 添加绘图到指定分组
   * 示例: layer对像
   * @param geometry 绘图对象
   * @param idCustom 绘图分组的id名称 默认值draw
   */
  addGeometry(geometry, idCustom) {
    if (idCustom === undefined || idCustom === null || idCustom === "") {
      idCustom = "draw";
    }
    if (this.layerGroups[idCustom] === undefined) {
      this.layerGroups[idCustom] = window.L.featureGroup().addTo(this.map);
    }
    this.layerGroups[idCustom].addLayer(geometry);
  }

  /**
   * 添加绘图到指定分组
   * @param geometry 绘图对象
   * @param idCustom 绘图分组的id名称, 为空时查找所有的匹配
   */
  removeGeometry(geometry, idCustom) {
    if (idCustom !== null) {
      if (this.layerGroups[idCustom] !== undefined) {
        if (this.layerGroups[idCustom].hasLayer(geometry)) {
          this.layerGroups[idCustom].removeLayer(geometry);
        }
      }
    } else {
      for (let idCustom in this.layerGroups) {
        if (this.layerGroups[idCustom].hasLayer(geometry)) {
          this.layerGroups[idCustom].removeLayer(geometry);
          break;
        }
      }
    }
  }

  /**
   *删除绘图分组中的所有图形
   */
  removeAllGeometry() {
    for (let idCustom in this.layerGroups) {
      this.layerGroups[idCustom].clearLayers();
      if (idCustom !== "draw") {
        this.map.removeLayer(this.layerGroups[idCustom]);
        delete this.layerGroups[idCustom];
      }
    }
    if (this.areaMeasure && this.areaMeasure.layers) {
      this.areaMeasure.layers.clearLayers();
    }
    this.stopDrawState();
  }

  /**
   *删除指定绘图分组的所有图形
   * @idCustom 绘图分组的id名称
   */
  removeGeometryByIdCustom(idCustom) {
    if (this.layerGroups[idCustom] !== undefined) {
      this.layerGroups[idCustom].clearLayers();
      if (idCustom !== "draw") {
        this.map.removeLayer(this.layerGroups[idCustom]);
        delete this.layerGroups[idCustom];
      }
    }
  }

  /**
   * 获取指定分组里的图形
   * @param idCustom 绘图分组的id名称 默认值draw
   */
  getGeometry(idCustom) {
    if (idCustom === undefined || idCustom === null || idCustom === "") {
      idCustom = "draw";
    }
    let geoJSON = "";
    if (this.layerGroups[idCustom] !== undefined && this.layerGroups[idCustom] !== null) {
      let layerGroup = this.layerGroups[idCustom];
      geoJSON = layerGroup.toGeoJSON();
    }
    return geoJSON;
  }

  /**
   * 创建geoJson对象图形
   * 示例：geoJson: {'type': 'Feature', 'properties': {}, 'geometry': {'type': 'Polygon', 'coordinates': [[[109,30],[115,30],[115,32],[109,32],[109,30]]]}}
   *       options: {color:"#fb030f",dashArray:"1",fillColor:"#000000",fillOpacity:0.5,weight:2}
   * @param geoJson geoJson格式的字符串对象
   * @param options 属性设置，主要是样式的设置
   * @returns geoJson对象图形
   */
  creareGeometryByGeoJson(geoJson, options) {
    if (options !== null && options !== undefined) {
      let greenIcon = window.L.icon({
        iconUrl: "images/point_position.png",
        iconSize: [40, 42],
        iconAnchor: [22, 42]
      });
      options.icon = greenIcon;
      options.markersInheritOptions = true;
      return window.L.geoJson(geoJson, options);
    } else {
      return window.L.geoJson(geoJson);
    }
  }

  /**
   * 创建对象图形
   * 示例：type: polygon
   *       geoJson: [[30,109],[30,115],[32,115],[32,109],[30,109]]
   *       options: {color:"#fb030f",dashArray:"1",fillColor:"#000000",fillOpacity:0.5,weight:2}
   * @param type 图形类型： polygon、polyline、point
   * @param latLngs 坐标集合
   * @param options 属性设置，主要是样式的设置
   * @returns {*}
   */
  creareGeometryByLatLngs(type, latLngs, options) {
    let geometry = null;
    switch (type) {
      case "polygon":
        geometry = window.L.polygon(latLngs, options);
        break;
      case "polyline":
        geometry = window.L.polyline(latLngs, options);
        break;
      case "point":
        geometry = window.L.point(latLngs, options);
        break;
    }
    return geometry;
  }

  /**
   * 创建marker对象
   * 示例：latlng: [30, 109]
   *       options: {opacity:1}
   * @param latlng 坐标
   * @param options 属性设置
   * @returns marker对象
   */
  createMarker(latlng, options) {
    if (options !== null && options !== undefined) {
      return window.L.marker(latlng, options);
    } else {
      return window.L.marker(latlng);
    }
  }

  /**
   *删除指定绘图分组的所有图形  以及清除正在绘制的图形
   * @idCustom 绘图分组的id名称
   */
  removeAllLayerFromMeasure() {
    if (this.areaMeasure && this.areaMeasure.layers) {
      this.areaMeasure.layers.clearLayers();
      this.map
        .off("click", this.areaMeasure.click)
        .off("mousemove", this.areaMeasure.mousemove)
        .off("contextmenu", this.areaMeasure.contextmenu);
    }
  }

  // /**
  //  * 绘图方法
  //  * @callbackFun 回调函数 参数是draw:created的参数 默认值null（传入null时 绘图坐标自动添加到draw的分组）
  //  * @geometryType 绘图类型 polygon-面 polyline-线 rectangle-矩形 circle-圆
  //  * @drawType 绘图所属分类名称 默认值null（传入null时 类型为draw）
  //  * @geometryOptions 绘图图形样式
  //  */
  // startEditDraw(callbackFun, geometryType, idCustom, geometryOptions) {
  //   let _this = this;
  //
  //   switch (geometryType.toLowerCase()) {
  //     case "polygon":
  //       this.map.editTools.options.drawingCursor = "url(images/cur/cursor-draw-polygon.cur),default";
  //       this.map.editTools.startPolygon(null, geometryOptions);
  //       this.map.on("editable:vertex:dragend", vertexFun);
  //       this.map.on("editable:vertex:deleted", vertexFun);
  //       this.map.once("editable:drawing:end", createdFun);
  //       this.map.doubleClickZoom.disable();
  //
  //       break;
  //     case "polyline":
  //       this.map.editTools.options.drawingCursor = "url(images/cur/cursor-draw-polyline.cur),default";
  //       this.map.editTools.startPolyline(null, geometryOptions);
  //       this.map.on("editable:vertex:dragend", vertexFun);
  //       this.map.on("editable:vertex:deleted", vertexFun);
  //       this.map.once("editable:drawing:end", createdFun);
  //       this.map.doubleClickZoom.disable();
  //
  //       break;
  //     case "rectangle":
  //       this.map.editTools.options.drawingCursor = "url(images/cur/cursor-draw-react.cur),default";
  //       this.map.editTools.startRectangle(null, geometryOptions);
  //       this.map.on("editable:vertex:dragend", vertexFun);
  //       this.map.on("editable:vertex:deleted", vertexFun);
  //       this.map.once("editable:drawing:end", createdFun);
  //       break;
  //     case "circle":
  //       this.map.editTools.options.drawingCursor = "url(images/cur/cursor-draw-cirle.cur),default";
  //       this.map.editTools.startCircle(null, geometryOptions);
  //       this.map.on("editable:vertex:dragend", vertexFun);
  //       this.map.on("editable:vertex:deleted", vertexFun);
  //       this.map.once("editable:drawing:end", createdFun);
  //       break;
  //     case "point": {
  //       this.map.editTools.options.drawingCursor = "url(images/cur/cursor-draw-d.cur),default";
  //       let greenIcon = window.L.icon({
  //         iconUrl: "images/point_position.png",
  //         iconSize: [40, 42],
  //         iconAnchor: [22, 42]
  //       });
  //       geometryOptions["icon"] = greenIcon;
  //       let marker = this.map.editTools.startMarker(null, geometryOptions);
  //       this.map.once("editable:drawing:end", createdFun);
  //       // 找不到marker的拖动的监听事件 使用原生的拖动事件
  //       marker.on("dragend", markerDragend);
  //       break;
  //     }
  //   }
  //
  //   function createdFun(e) {
  //     _this.map.editTools.options.drawingCursor = "crosshair";
  //     document.getElementById(_this.mapTarget).style.cursor = "";
  //     //_this.map.off("editable:drawing:end", createdFun);
  //     e.layer.layerType = geometryType;
  //     // 矩形有bug 特殊处理 矩形绘制的时候 只单击地图 不拉框 会出现返回的矩形是4个相同的点
  //     if (e.layer.layerType === "rectangle") {
  //       let bounds = e.layer._bounds;
  //       if (bounds._northEast != undefined && bounds._southWest != undefined) {
  //         if (bounds._northEast.lat === bounds._southWest.lat && bounds._northEast.lng === bounds._southWest.lng) {
  //           _this.drawCreated(e, callbackFun, idCustom);
  //         }
  //       }
  //     } else {
  //       _this.drawCreated(e, callbackFun, idCustom);
  //     }
  //
  //     if (!_this.map.doubleClickZoom._enabled) {
  //       _this.map.once("dblclick", () => {
  //         _this.map.doubleClickZoom.enable();
  //       });
  //     }
  //   }
  //
  //   function vertexFun(e) {
  //     _this.map.editTools.options.drawingCursor = "crosshair";
  //     document.getElementById(_this.mapTarget).style.cursor = "";
  //
  //     _this.drawCreated(e, callbackFun, idCustom);
  //   }
  //
  //   function markerDragend(e) {
  //     _this.map.editTools.options.drawingCursor = "crosshair";
  //     document.getElementById(_this.mapTarget).style.cursor = "";
  //     _this.drawCreated(e, callbackFun, idCustom);
  //   }
  // }

  // 关闭绘图的状态
  stopDrawState() {
    if (this._drawLayerState) {
      this._drawLayerState.disable();
      this.map.off("pm:drawend");
      this.map.off("pm:create");
    }
    this.stopDrawRectPoint();
    if (this.areaMeasure) {
      this.map
        .off("click", this.areaMeasure.click)
        .off("mousemove", this.areaMeasure.mousemove)
        .off("contextmenu", this.areaMeasure.contextmenu);
    }
  }

  /**
   * 绘制完成后的回调
   * @param event
   */
  // drawCreated(event, callbackFun, idCustom) {
  //   // 编辑绘图监听没有layerType 在传入的属性中添加了type
  //   const type = event.layer.layerType;
  //
  //   let layer = event.layer;
  //
  //   this.addGeometry(event.layer, idCustom);
  //
  //   let jsonGeometry = null;
  //   switch (type) {
  //     case "point":
  //     case "polyline":
  //     case "polygon": {
  //       let geoJSON = layer.toGeoJSON();
  //       jsonGeometry = geoJSON.geometry;
  //       break;
  //     }
  //     case "rectangle": {
  //       let bounds = event.layer._bounds;
  //       if (bounds._northEast.lat === bounds._southWest.lat && bounds._northEast.lng === bounds._southWest.lng) {
  //         jsonGeometry = {
  //           type: "Polygon",
  //           coordinates: []
  //         };
  //         event.layer.remove();
  //       } else {
  //         let geoJSON = layer.toGeoJSON();
  //         jsonGeometry = geoJSON.geometry;
  //       }
  //       break;
  //     }
  //     case "circle": {
  //       let points = [];
  //       let DOUBLE_PI = Math.PI * 2;
  //       let angle = 0.0;
  //       let projectedCentroid;
  //       let radiusX;
  //       let radiusY;
  //       let point;
  //       let project;
  //       let unproject;
  //       let CIRCLE_SECTIONS_COUNT = 100;
  //
  //       project = this.map.latLngToLayerPoint.bind(this.map);
  //       unproject = this.map.layerPointToLatLng.bind(this.map);
  //       radiusX = layer._radius;
  //       radiusY = layer._radiusY;
  //       projectedCentroid = project(layer._latlng);
  //       const vertices = CIRCLE_SECTIONS_COUNT;
  //       jsonGeometry = {
  //         type: "Polygon",
  //         coordinates: [[]]
  //       };
  //
  //       for (let i = 0; i < vertices - 1; i++) {
  //         angle -= DOUBLE_PI / vertices; // clockwise
  //         point = new window.L.Point(
  //           projectedCentroid.x + radiusX * Math.cos(angle),
  //           projectedCentroid.y + radiusY * Math.sin(angle)
  //         );
  //         if (i > 0 && point.equals(points[i - 1])) {
  //           continue;
  //         }
  //         points.push(unproject(point));
  //         jsonGeometry.coordinates[0].push([points[i].lng, points[i].lat]);
  //       }
  //       break;
  //     }
  //   }
  //   if (callbackFun !== null && callbackFun !== undefined) {
  //     callbackFun(jsonGeometry, event);
  //   }
  // }

  startMeasure(callFun, drawType, options) {
    drawType = drawType || "polygon";

    options = options || this.measureStyle;
    options["showMeasurements"] = true;
    options["callFun"] = callFun;
    /* options['measurementOptions'] = {
            unit: 2
        }; */
    let _this = this;

    if (!_this.areaMeasure) _this.areaMeasure = createAreaMeasure();

    _this.areaMeasure.init(drawType, options);
    function createAreaMeasure() {
      const areaMeasure = {
        points: [],
        points_click: [],
        layers: window.L.layerGroup(),
        layer: null,
        editable: true,
        drawType: "polygon",
        options: {},
        callFun: null,
        init: function (drawType, options) {
          this.points = []; // 临时点数组，包括点击点、移动点
          this.points_click = []; // 最终点数组，只有点击点
          this.layer = null;
          this.drawType = drawType;
          this.options = options;
          this.callFun = options.callFun;
          if (drawType === "polygon") {
            document.getElementById(_this.mapTarget).style.cursor = "url(images/cur/cursor-cl-m.cur),move";
          } else if (drawType === "polyline") {
            document.getElementById(_this.mapTarget).style.cursor = "url(images/cur/cursor-cl-x.cur),move";
          }
          _this.map.on("click", this.click).on("contextmenu", this.contextmenu);
        },
        click: function (e) {
          let _areaMeasure = _this.areaMeasure;

          _this.map.doubleClickZoom.disable();
          // 添加点信息
          _areaMeasure.points.push(e.latlng);
          _areaMeasure.points_click.push(e.latlng);
          // 添加面
          _this.map.on("mousemove", _areaMeasure.mousemove);
          _areaMeasure.layers.addTo(_this.map);
          _this.updatepointerEvents(0);
        },
        mousemove: function (e) {
          let _areaMeasure = _this.areaMeasure;

          _areaMeasure.points.push(e.latlng);

          if (_areaMeasure.layer) _areaMeasure.layers.removeLayer(_areaMeasure.layer);

          if (_areaMeasure.drawType === "polygon") {
            _areaMeasure.layer = window.L.polygon(_areaMeasure.points, _areaMeasure.options);
          } else if (_areaMeasure.drawType === "polyline") {
            _areaMeasure.options["measurementOptions"] = { imperial: false };
            _areaMeasure.layer = window.L.polyline(_areaMeasure.points, _areaMeasure.options);
          }

          _areaMeasure.layer.addTo(_areaMeasure.layers);
          _areaMeasure.points.pop();

          _this.updatepointerEvents(0);
        },
        // 结束
        contextmenu: function () {
          _this.updatepointerEvents(1);
          document.getElementById(_this.mapTarget).style.cursor = "";
          let _areaMeasure = _this.areaMeasure;
          _areaMeasure.layers.clearLayers();

          if (_areaMeasure.drawType === "polygon") {
            _areaMeasure.layer = window.L.polygon(_areaMeasure.points_click, _areaMeasure.options);
          } else if (_areaMeasure.drawType === "polyline") {
            _areaMeasure.options["measurementOptions"] = { imperial: false };
            _areaMeasure.layer = window.L.polyline(_areaMeasure.points_click, _areaMeasure.options);
          }

          _areaMeasure.layer.addTo(_areaMeasure.layers);
          // 绘图可编辑  如果map初始化的时候没设置 editable: true 就会报错
          _areaMeasure.layer.enableEdit();
          // 绘图编辑时 面积实时变化
          _this.map.on(
            "editable:vertex:drag editable:vertex:deleted",
            _areaMeasure.layer.updateMeasurements,
            _areaMeasure.layer
          );

          _this.map
            .off("click", _areaMeasure.click)
            .off("mousemove", _areaMeasure.mousemove)
            .off("contextmenu", _areaMeasure.contextmenu);
          if (_areaMeasure.callFun) _areaMeasure.callFun(_areaMeasure.layer);
        }
      };
      return areaMeasure;
    }
  }

  /**
   * 拉框查询的点与矩形的绘制
   * @param callbackFun 回调方法，返回坐标
   * @param isMultiple 是执行多次，还是一次， 默认: false
   * @param isPoint 是否是点选， 默认: false
   * @param imgUrl 根据不同的业务设置不同的地图鼠标状态
   * @param drawStyle 绘制样式，可选，有默认
   */
  startDrawRectPoint(callbackFun, isMultiple, isPoint, imgUrl, drawStyle) {
    let that = this;
    this._mapView.setMapCursor(imgUrl);
    that.stopDrawRectPoint();
    that.drawRectPointHandler = {
      tmprect: null,
      latlngs: [],
      drawStyle: drawStyle || { color: "#3385ff", weight: 3, dashArray: 1 },
      onDown: function (e) {
        if (e.originalEvent.button === 0) {
          // 0为鼠标左键 1为滚轮按压 2为鼠标右键
          that.map.dragging.disable();
          let this_ = that.drawRectPointHandler;
          if (this_.tmprect !== null) {
            this_.tmprect.remove();
            this_.tmprect = null;
          }
          // 左上角坐标
          this_.latlngs[0] = [e.latlng.lat, e.latlng.lng];
          // 开始绘制，监听鼠标移动事件
          if (isPoint === undefined || !isPoint) {
            that.map.on("mousemove", this_.onMove);
          }
          that.map.on("mouseup", this_.onUp);
        }
      },
      onMove: function (e) {
        let this_ = that.drawRectPointHandler;
        this_.latlngs[1] = [e.latlng.lat, e.latlng.lng];
        if (this_.tmprect === null) {
          this_.tmprect = window.L.rectangle(this_.latlngs, this_.drawStyle).addTo(that.map);
          that.updatepointerEvents(0);
        } else {
          this_.tmprect.setBounds(window.L.latLngBounds(this_.latlngs[0], this_.latlngs[1]));
        }
      },
      onUp: function (e) {
        if (e.originalEvent.button === 0) {
          // 0为鼠标左键 1为滚轮按压 2为鼠标右键
          // 矩形绘制完成，移除临时矩形，并停止监听鼠标移动事件
          let this_ = that.drawRectPointHandler;
          if (this_.tmprect !== null) {
            this_.tmprect.remove();
            this_.tmprect = null;
          }
          that.map.off("mousemove", this_.onMove);
          that.map.off("mouseup", this_.onUp);
          that.updatepointerEvents(1);
          if (isMultiple === undefined || isMultiple === null || !isMultiple) {
            that.map.off("mousedown", this_.onDown);
            that._mapView.setMapCursor("");
          }
          that.map.dragging.enable();
          // 右下角坐标
          this_.latlngs[1] = [e.latlng.lat, e.latlng.lng];
          let p1 = [this_.latlngs[0][1], this_.latlngs[0][0]];
          let p3 = [e.latlng.lng, e.latlng.lat];
          if (Math.abs(p3[0] - p1[0]) < 0.000000000000001 && Math.abs(p3[1] - p1[1]) < 0.000000000000001) {
            callbackFun({ type: "Point", coordinates: p1 });
          } else {
            callbackFun({
              type: "Polygon",
              coordinates: [[[p3[0], p1[1]], p3, [p1[0], p3[1]], p1, [p3[0], p1[1]]]]
            });
          }
        }
      }
    };
    that.map.on("mousedown", that.drawRectPointHandler.onDown);
  }
  // 关闭拉框查询的状态
  stopDrawRectPoint() {
    if (this.drawRectPointHandler !== undefined && this.drawRectPointHandler !== null) {
      this.map.off("mousemove", this.drawRectPointHandler.onMove);
      this.map.off("mouseup", this.drawRectPointHandler.onUp);
      this.map.off("mousedown", this.drawRectPointHandler.onDown);
      this.drawRectPointHandler = null;
      this._mapView.setMapCursor("");
    }
  }

  /**
   * 修改绘图时marker中图形对象的鼠标状态
   * @type 1-有效 0-无效
   */
  updatepointerEvents(type) {
    let cs = document.getElementsByClassName("leaflet-interactive");
    for (let i = 0; i < cs.length; i++) {
      if (type === 1) {
        cs[i].style.pointerEvents = "auto";
      } else {
        cs[i].style.pointerEvents = "none";
      }
    }
  }

  /**
   * 定位图形绘制 并闪烁
   * @features geojson对象 支持FeatureCollection GeometryCollection feature polygon的GeoJson格式
   * @idCustom 绘图结果保存到layer分组的名称 默认result
   * @callFun 回调方法 返回绘制的layer对象
   * @fillStyle 填充样式 颜色与宽度 例：{lineColor: 'rgba(0, 0, 255, 1)', lineWidth: '2px'}
   */
  drawAndFlytoGeometry(feature, idCustom = "result", callFun = null, drawStyle = null, fillStyle = null) {
    if (drawStyle === null || drawStyle === undefined || drawStyle === "") {
      drawStyle = this.drawLocationStyle;
    }
    if (fillStyle === null || fillStyle === undefined) {
      fillStyle = this.fillLocationStyle;
    }
    if (fillStyle !== null && Object.keys(fillStyle).length !== 0) {
      drawStyle = this._getFillStyle(fillStyle, drawStyle);
    }
    let tempLayer = this.creareGeometryByGeoJson(feature, drawStyle);
    this.addGeometry(tempLayer, idCustom);
    let bounds = tempLayer.getBounds();
    // 点数据定位 使用flyto 不改变比例尺
    if (bounds._northEast.lat === bounds._southWest.lat && bounds._northEast.lng === bounds._southWest.lng) {
      this.map.flyTo([bounds._southWest.lat, bounds._southWest.lng]);
    } else {
      this.map.fitBounds(bounds, {
        paddingTopLeft: [200, 200],
        paddingBottomRight: [200, 200]
      });
    }

    // 闪烁效果
    let count = 0;
    let fillOpacity = 0.3;
    let opacity = 1;
    if (drawStyle.fillOpacity !== undefined) {
      fillOpacity = drawStyle.fillOpacity;
    }
    if (drawStyle.opacity !== undefined) {
      opacity = drawStyle.opacity;
    }
    let mylet = setInterval(() => {
      if (count < 4) {
        if (count % 2 === 0) {
          tempLayer.setStyle({ opacity: 0, fillOpacity: 0 });
        } else {
          tempLayer.setStyle({ opacity: opacity, fillOpacity: fillOpacity });
        }
        count++;
      } else {
        this.map.stop();
        clearInterval(mylet);
      }
    }, 200);
    if (callFun) callFun(tempLayer, feature);
  }
  /**
   * 图形绘制 并闪烁 (不定位)
   * @features geojson对象 支持FeatureCollection GeometryCollection feature polygon的GeoJson格式
   * @idCustom 绘图结果保存到layer分组的名称 默认result
   */
  locationDrawNotPos(feature, idCustom = "result") {
    let tempLayer = this.creareGeometryByGeoJson(feature, this.drawLocationStyle);
    this.addGeometry(tempLayer, idCustom);
    // 闪烁效果
    let count = 0;
    let mylet = setInterval(() => {
      if (count < 4) {
        if (count % 2 === 0) {
          tempLayer.setStyle({ opacity: 0 });
        } else {
          tempLayer.setStyle({ opacity: 1 });
        }
        count++;
      } else {
        this.map.stop();
        clearInterval(mylet);
      }
    }, 200);
    return tempLayer;
  }

  locationDraw2Marker(lat, lng, idCustom = "result") {
    let marker = this.createMarker([lat, lng]);
    this.addGeometry(marker, idCustom);
    this.map.flyTo(marker.getLatLng());
  }
  /**
   * 生成svg格式的填充样式
   * @param {*} fillStyle
   * @param {*} drawStyle
   * @returns
   */
  _getFillStyle(fillStyle, drawStyle) {
    let lineColor = fillStyle.lineColor || "rgba(0, 0, 255, 1)";
    let lineWidth = fillStyle.lineWidth || "2px";
    let style =
      "stroke-linecap: square;stroke-linejoin: miter;overflow: inherit;stroke: " +
      lineColor +
      ";stroke-width: " +
      lineWidth +
      ";";
    let width = 10;
    let height = 10;
    let lineType = "left";
    if (fillStyle.lineSpacing !== undefined) {
      width = height = fillStyle.lineSpacing;
    }
    if (fillStyle.lineType !== undefined) {
      lineType = fillStyle.lineType;
    }
    let svgLine = "";
    if (lineType === "left") {
      svgLine =
        '<line x1="' +
        width / 2 +
        '" y1="' +
        0 +
        '" x2="' +
        width / 2 +
        '" y2="' +
        height +
        '" style="' +
        style +
        '"></line>';
    } else if (lineType === "right") {
      svgLine =
        '<line x1="' +
        0 +
        '" y1="' +
        height / 2 +
        '" x2="' +
        width +
        '" y2="' +
        height / 2 +
        '" style="' +
        style +
        '"></line>';
    } else {
      svgLine =
        '<line x1="' +
        width / 2 +
        '" y1="' +
        0 +
        '" x2="' +
        width / 2 +
        '" y2="' +
        height +
        '" style="' +
        style +
        '"></line>';
      svgLine +=
        '<line x1="' +
        0 +
        '" y1="' +
        height / 2 +
        '" x2="' +
        width +
        '" y2="' +
        height / 2 +
        '" style="' +
        style +
        '"></line>';
    }
    let svgStr =
      '<svg xmlns:svg="http://www.w3.org/2000/svg" xmlns="http://www.w3.org/2000/svg" width="' +
      width +
      '" height="' +
      height +
      '" style="overflow: inherit;">' +
      svgLine +
      "</svg>";
    let retStyle = {};
    for (let key in drawStyle) {
      retStyle[key] = drawStyle[key];
    }
    retStyle["fill"] = "url(data:image/svg+xml;base64," + window.btoa(svgStr) + ")";
    retStyle["fillOpacity"] = 1;
    retStyle["fillColor"] = "rgba(0,0,0,0)";
    retStyle["patternDegree"] = 45;
    return retStyle;
  }

  startDraw(callbackFun, geometryType, idCustom, geometryOptions, mapTarget) {
    this.stopDrawState();

    geometryOptions = { ...this.drawStyle, ...geometryOptions };

    let _options = {
      templineStyle: geometryOptions, // 临时线样式
      hintlineStyle: { color: geometryOptions.color, dashArray: "5,5" }, // 临时线最终移动点样式
      pathOptions: geometryOptions // 最终结果绘制
    };

    switch (geometryType.toLowerCase()) {
      case "point":
        this._drawLayerState = new window.L.PM.Draw.Marker(this.map);
        // 绘制marker样式
        _options = {
          markerStyle: {
            // 支持拖动
            draggable: true,
            icon: window.L.icon({ iconUrl: markerUrl, iconSize: [32, 32], iconAnchor: [16, 32] })
          }
        };
        break;
      case "polygon":
        this._drawLayerState = new window.L.PM.Draw.Polygon(this.map);
        break;
      case "polyline":
        this._drawLayerState = new window.L.PM.Draw.Line(this.map);
        break;
      case "rectangle":
        this._drawLayerState = new window.L.PM.Draw.Rectangle(this.map);
        break;
      case "circle":
        this._drawLayerState = new window.L.PM.Draw.Circle(this.map);
        break;
    }
    this._drawLayerState.enable(_options);

    let _this = this;
    _this.map.on("pm:create", createdFun);
    _this.map.on("pm:drawend", drawStop);

    function drawStop() {
      _this.map.off("pm:drawend", drawStop);
    }

    function createdFun(e) {
      // marker是连点状态 绘制一次就停止
      if (geometryType.toLowerCase() === "point") {
        _this._drawLayerState.disable();
      }
      _this.map.off("pm:drawend", drawStop);
      _this.map.off("pm:create", createdFun);
      drawCreated(e, callbackFun, idCustom);
    }

    function drawCreated(event, callbackFun, idCustom) {
      // 编辑绘图监听没有layerType 在传入的属性中添加了type
      const type = event.shape;

      let layer = event.layer;

      layer.remove();

      _this.addGeometry(event.layer, idCustom);

      let jsonGeometry = null;
      switch (type.toLowerCase()) {
        case "marker":
        case "line":
        case "polygon": {
          let geoJSON = layer.toGeoJSON();
          jsonGeometry = geoJSON.geometry;
          break;
        }
        case "rectangle": {
          let bounds = event.layer._bounds;
          if (bounds._northEast.lat === bounds._southWest.lat && bounds._northEast.lng === bounds._southWest.lng) {
            jsonGeometry = {
              type: "Polygon",
              coordinates: []
            };
            event.layer.remove();
          } else {
            let geoJSON = layer.toGeoJSON();
            jsonGeometry = geoJSON.geometry;
          }
          break;
        }
        case "circle": {
          let points = [];
          let DOUBLE_PI = Math.PI * 2;
          let angle = 0.0;
          let projectedCentroid;
          let radiusX;
          let radiusY;
          let point;
          let project;
          let unproject;
          let CIRCLE_SECTIONS_COUNT = 100;

          project = _this.map.latLngToLayerPoint.bind(_this.map);
          unproject = _this.map.layerPointToLatLng.bind(_this.map);
          radiusX = layer._radius;
          radiusY = layer._radiusY;
          projectedCentroid = project(layer._latlng);
          const vertices = CIRCLE_SECTIONS_COUNT;
          jsonGeometry = {
            type: "Polygon",
            coordinates: [[]]
          };

          for (let i = 0; i < vertices - 1; i++) {
            angle -= DOUBLE_PI / vertices; // clockwise
            point = new window.L.Point(
              projectedCentroid.x + radiusX * Math.cos(angle),
              projectedCentroid.y + radiusY * Math.sin(angle)
            );
            if (i > 0 && point.equals(points[i - 1])) {
              continue;
            }
            points.push(unproject(point));
            jsonGeometry.coordinates[0].push([points[i].lng, points[i].lat]);
          }
          break;
        }
      }
      if (callbackFun !== null && callbackFun !== undefined) {
        callbackFun(jsonGeometry, event);
      }
    }
  }

  enableEdit(layer, isEnable, callbackFun) {
    if (isEnable) {
      layer.pm.enable({
        allowSelfIntersection: false
      });
      layer.on("pm:edit", function (e) {
        if (callbackFun !== null && callbackFun !== undefined) {
          callbackFun(layer, e);
        }
      });
    } else {
      layer.pm.disable();
      layer.off("pm:edit");
    }
  }
}
